package main

import (
	"errors"
	"github.com/gofiber/fiber/v2"
	"log"
	"time"
)

type TestCommand struct {
	Command    string `json:"cmd"`
	SequenceId int64  `json:"sequenceId"`
}

var currentTest *TestCommand

var testChannel chan *TestCommand
var testStopChannel chan bool

func setTestCmd(c *fiber.Ctx) error {
	var testCmd TestCommand
	if err := c.BodyParser(&testCmd); nil != err {
		return failureResponse(c, "-1", err.Error(), 400)
	} else if nil == currentScene {
		return failureResponse(c, "1", "not in a working scene", 400)
	} else {
		currentTest = &testCmd
		switch testCmd.Command {
		case "START":
			log.Println("START ...")
			if nil != currentTest {
				return failureResponse(c, "2", "A test already started", 403)
			}
			if e := recogTaskProc(&testCmd); nil != e {
				return failureResponse(c, "3", e.Error(), 409)
			}
		case "STOP":
			log.Println("STOP ...")
		case "CANCEL":
			log.Println("CANCEL ...")
		default:
			log.Println("UNKNOWN ...")
		}
		return successResponse(c, "Updated successfully.")
	}
}

func pullUpTask(seq int64) error {
	var tk = time.NewTicker(time.Second * 3)
	defer func() {
		tk.Stop()
	}()
	for {
		select {
		case <-testStopChannel:
			return nil
		case <-tk.C:
			log.Println("===")
		}
	}
}

func standJumpTask(seq int64) error {
	var tk = time.NewTicker(time.Second * 3)
	defer func() {
		tk.Stop()
	}()
	for {
		select {
		case <-testStopChannel:
			return nil
		case <-tk.C:
			log.Println("===")
		}
	}
}

func sitUpsTask(seq int64) error {
	var tk = time.NewTicker(time.Second * 3)
	defer func() {
		tk.Stop()
	}()
	for {
		select {
		case <-testStopChannel:
			return nil
		case <-tk.C:
			log.Println("===")
		}
	}
}

func raceTask(seq int64) error {
	var tk = time.NewTicker(time.Second * 3)
	defer func() {
		tk.Stop()
	}()
	for {
		select {
		case <-testStopChannel:
			return nil
		case <-tk.C:
			log.Println("===")
		}
	}
}

func recogTaskProc(cmd *TestCommand) error {
	testStopChannel = make(chan bool)
	if nil == currentScene {
		return errors.New("invalid scene state")
	}
	if nil == currentScene.proc {
		return errors.New("invalid scene mode processor")
	}
	currentTest = cmd
	go func() {
		defer func() {
			close(testStopChannel)
			currentTest = nil
		}()
		if e := currentScene.proc(cmd.SequenceId); nil != e {
			log.Println(">>>>", e)
		}
	}()
	return nil
}

func init() {
	currentTest = nil
	testChannel = nil
	testStopChannel = nil
}
